/***************************************************************
 * Name:      wxiscanimageprofile.h
 * Purpose:   ...
 * Author:    Daniel Nell (daniel.nell@nellresearch.de)
 * Created:   2009-01-12
 * Copyright: Daniel Nell (www.nellresearch.de)
 * License:   GPL
 **************************************************************/

#ifndef WXISCANIMAGEPROFILE_H_INCLUDED
#define WXISCANIMAGEPROFILE_H_INCLUDED



/////////////////////////////////////////////////////////////////////////////
//  Class wxIScanImageProfile
//
/// \brief ...
///
/// ...
class wxIScanImageProfile //: public wxObject
{
  public:
    /// \brief Standard constructor
    ///
    /// ...
    wxIScanImageProfile( int nProfileIndex, int nResolution )
    {
        LoadConfig( nProfileIndex, nResolution );
    }

    /// Virtual destructor.
    virtual ~wxIScanImageProfile()
    {
        SaveConfig();
    }

    /// \brief Load configuration from the configuration object/file.
    ///
    /// ...
    virtual void LoadConfig( int nProfileIndex, int nResolution );

    /// \brief Save configuration to the configuration object/file.
    ///
    /// ...
    virtual void SaveConfig();

  public:
    virtual int GetIndex(){ return m_nProfileIndex; }       ///<  Contains the profile's index number.
    virtual wxString GetName(){ return m_strProfileName; }  ///<  Returns the profile name.
    virtual wxImage ApplyToImage( wxImage oImage );         ///<  Apply changes to the give image.

  private:
    int m_nProfileIndex;                                    ///<  Contains the profile's index number.
    wxString m_strProfileName;                              ///<  Contains the profile's name.
    bool bCrop;                                             ///<  Crop image?
    int x;                                                  ///<  x-coordinate in pixels of the upper left corner of the cropping region.
    int y;                                                  ///<  y-coordinate in pixels of the upper left corner of the cropping region.
    int w;                                                  ///<  Width of the cropping region in pixels.
    int h;                                                  ///<  Height of the cropping region in pixels.
    bool bGreyScale;                                        ///<  Convert image to greyscale?
    bool bMonoChrome;                                       ///<  Convert image to monochrome?
    int nMonoThreshold;                                     ///<  Treshhold value to segment black and white while converting to greyscale.
    int nRotate90Right;                                     ///<  Rotate image 90° to the right # of times.
};


/////////////////////////////////////////////////////////////////////////////
//  Global image manipulation functions
//
/// \brief Convert an image to monochrome (black and white).
///
/// \param oSrcImage        the image to convert
/// \param nMonoThreshold   ...
wxImage Image2Mono( const wxImage &oSrcImage, int nMonoThreshold= 128 );

/// \brief Modify brightness, contrast, gamma value, red, green
///        and blue channel of an image.
///
/// \param oSrcImage        the image to convert
/// \param nBrightness      ...
/// \param nContrast        ...
/// \param nGamma           ...
/// \param nRed             ...
/// \param nGreen           ...
/// \param nBlue            ...
wxImage EnhenceColours( const wxImage &oSrcImage,
                        int nBrightness= 0, int nContrast= 0, int nGamma= 100,
                        int nRed= 0, int nGreen= 0, int nBlue= 0 );

#endif // WXISCANIMAGEPROFILE_H_INCLUDED
