/***************************************************************
 * Name:      wxpoppler.h
 * Purpose:   Code for pdf file access (for viewing pdf files)
 * Author:    Daniel Nell (daniel.nell@nellresearch.de)
 * Created:   2008-07-08
 * Copyright: Daniel Nell (www.nellresearch.de)
 * License:
 **************************************************************/

#ifndef WXPOPPLER_H
#define WXPOPPLER_H

#include <poppler.h>
#if __WXPOPPLER_DYNAMIC__
#   include <wx/dynlib.h>
#endif // __WXPOPPLER_DYNAMIC__
#include <wx/image.h>


/////////////////////////////////////////////////////////////////////////////
//  Defines
//
#if __WXPOPPLER_DYNAMIC__
#   define WXPOPPPLER_LIBPOPPLER    "poppler-glib"
#endif // __WXPOPPLER_DYNAMIC__


/////////////////////////////////////////////////////////////////////////////
//  Class wxPoppler
//
/// \brief A class to handle PDF files using libpoppler.
///
/// ...
class wxPoppler
{
  public:
    wxPoppler();                                                            ///< Standard constructor.
    virtual ~wxPoppler();                                                   ///< Virtual destructor.

  public:
    bool Open( const wxString& strFileName ){ return Open( NULL, 0, strFileName ); } ///< Opens a PDF file.
    bool Open( char *pcData, const int nLength ){ return Open( pcData, nLength, wxEmptyString ); } ///< Opens a PDF file in memory.
    int  GetPageCount(){ return poppler_document_get_n_pages( m_pPdfDocument ); } ///< ...
    bool SelectPage( int nIndex );                                          ///< ...
    bool RenderPage();                        ;                             ///< Render the PDF page and store the result in m_oImage.
    int  SetDpi( int nDpi ){ int nOldDpi= m_nDpi; m_nDpi= nDpi; return nOldDpi; } ///< ...
    wxImage GetImage(){ return m_oImage; }                                  ///< Get the rendered PDF page as wxImage object.

  protected:
    /// Opens a PDF stream from file or memory.
    bool Open( char *pcData, const int nLength, const wxString& strFileName );

  protected:
    wxImage m_oImage;                                                       ///< Image object rendered by RenderPage().
    int m_nDpi;                                                             ///< Holds the image resolution in dots per inch.
    PopplerPage *m_pPdfPage;                                                ///< A pointer to the current PDF page.
    PopplerDocument *m_pPdfDocument;                                        ///< A pointer to the current PDF document.

#if __WXPOPPLER_DYNAMIC__
  private:
    static wxDynamicLibrary m_oWxPopplerDynamicLibrary;

    typedef PopplerDocument * t_poppler_document_new_from_data( char *, int, const char *, GError ** );
    typedef PopplerDocument * t_poppler_document_new_from_file( const char *,const char *, GError ** );
    typedef PopplerPage * t_poppler_document_get_page( PopplerDocument *, int );
    typedef void t_poppler_page_render_to_pixbuf( PopplerPage *, int, int, int, int, double, int, GdkPixbuf * );
    typedef void t_poppler_page_get_size( PopplerPage *, double *, double * );
    typedef int t_poppler_document_get_n_pages( PopplerDocument * );

    static t_poppler_document_new_from_data *poppler_document_new_from_data;
    static t_poppler_document_new_from_file *poppler_document_new_from_file;
    static t_poppler_document_get_page *poppler_document_get_page;
    static t_poppler_page_render_to_pixbuf *poppler_page_render_to_pixbuf;
    static t_poppler_page_get_size *poppler_page_get_size;
    static t_poppler_document_get_n_pages *poppler_document_get_n_pages;
#endif // __WXPOPPLER_DYNAMIC__
};


#if wxUSE_PDF
/////////////////////////////////////////////////////////////////////////////
//  Enum wxBitmapTypePdf
//
/// \brief ...
///
/// ...
enum wxBitmapTypePdf
{
    wxBITMAP_TYPE_PDF = wxBITMAP_TYPE_ANY + 1047
};


/////////////////////////////////////////////////////////////////////////////
//  Class wxPDFHandler
//
/// \brief Image handler to handle PDF files in a way e. g. the BMP or JPEG
///        file handler does it.
///
/// ...
///
/// Note:  Only opening of PDF files is implemented, yet.
class wxPDFHandler : public wxImageHandler
{
  public:
    /// ...
    wxPDFHandler()
      : m_nPageCount( 0 )
    {
        m_name= wxT( "wxPDFHandler" );
        m_extension= wxT( "pdf" );
        m_type= wxBITMAP_TYPE_PDF;
        m_mime= wxT( "application/pdf" );
    }


#if wxUSE_STREAMS
  public:
    /// ...
    virtual bool LoadFile( wxImage* poImage, wxInputStream& oIStream, bool bVerbose= true, int nIndex= 0 );

    /// ...
    virtual bool SaveFile(wxImage* poImage, wxOutputStream& oOStream );

    /// ...
    virtual int GetImageCount( wxInputStream& oIStream ){ return m_nPageCount; }


  protected:
    /// ...
    virtual bool DoCanRead( wxInputStream& oIStream );
#endif // wxUSE_STREAMS

  private:
    int m_nPageCount;

  private:
    DECLARE_DYNAMIC_CLASS( wxPDFHandler )
};
#endif // wxUSE_PDF

#endif // WXPOPPLER_H
